@@ -92,7 +92,12 @@ function export_env_vars
  	fi
 	
 	echo "using pgpool-II at "$PGPOOL_PATH
+
+	export PGPOOL_VERSION=`$PGPOOL_PATH/bin/pgpool --version 2>&1`
+
 	export PGPOOL_INSTALL_DIR=$PGPOOL_PATH
+	# where to look for pgpool.conf.sample files.
+	export PGPOOLDIR=$PGPOOL_INSTALL_DIR/etc
 
 	PGPOOLLIB=${PGPOOL_INSTALL_DIR}/lib
 	if [ -z "$LD_LIBRARY_PATH" ];then
@@ -116,7 +121,8 @@ function print_info
 	echo ${CBLUE}"*************************"${CNORM}
 
 	echo "REGRESSION MODE          : "${CBLUE}$MODE${CNORM}
-	echo "PGPOOL-II                : "${CBLUE}$PGPOOL_PATH${CNORM}
+	echo "Pgpool-II version        : "${CBLUE}$PGPOOL_VERSION${CNORM}
+	echo "Pgpool-II install path   : "${CBLUE}$PGPOOL_PATH${CNORM}
 	echo "PostgreSQL bin           : "${CBLUE}$PGBIN${CNORM}
 	echo "PostgreSQL Major version : "${CBLUE}$PGVERSION${CNORM}
 	echo "pgbench                  : "${CBLUE}$PGBENCH_PATH${CNORM}
@@ -200,6 +206,13 @@ do
 	clean_all
 	timeout $TIMEOUT ./test.sh > $log/$i 2>&1
 	rtn=$?
+
+	check_segfault
+	if [ $? -eq 0 ];then
+		echo "fail: Segmentation fault detected" >> $log/$i
+		rtn=1
+	fi
+
 	if [ $rtn = 0 ];then
 		echo ${CGREEN}"ok."${CNORM}
 		ok=`expr $ok + 1`
